<cfsilent>
<!---
CF_WIKI
by Keith Gaughan <kmgaughan@eircom.net>

The wiki engine. Provides an interface to the wiki code for display and
editing.

Copyright (c) Keith Gaughan, 2004.
All Rights Reserved.
--->

<!--- Protect from independent invocation. --->
<cfif GetCurrentTemplatePath() EQ GetBaseTemplatePath()><cfabort></cfif>

<cfif ThisTag.HasEndTag EQ TRUE>
	<cfthrow message="This tag cannot have an end tag.">
</cfif>

<!--- START: Check parameters --->
<cfif Not IsDefined("ATTRIBUTES.pageDir")>
	<cfthrow message="Page directory not specified.">
</cfif>

<!--- get, set --->
<cfparam name="ATTRIBUTES.action" default="get" type="string">
<cfswitch expression="#ATTRIBUTES.action#">

	<!--- Retrieving the contents of a page. --->
	<cfcase value="get">
		<!--- display, editing --->
		<cfparam name="ATTRIBUTES.for" default="display" type="string">
		<!--- Variable to write timestamp to. --->
		<cfparam name="ATTRIBUTES.timestamp" type="variableName">
		<!--- Variable to write text to. --->
		<cfparam name="ATTRIBUTES.variable" type="variableName">
		<!--- Name of page. --->
		<cfparam name="ATTRIBUTES.name" type="string">
		<!--- Default text, if page doesn't exist. --->
		<cfparam name="ATTRIBUTES.default" default="" type="string">
		<!--- Should the text be formatted? --->
		<cfparam name="ATTRIBUTES.formatted" default="Yes" type="boolean">
	</cfcase>

	<!--- Saving the contents of a page. --->
	<cfcase value="set">
		<!--- Name of page. --->
		<cfparam name="ATTRIBUTES.name" type="string">
		<!--- New content for page. --->
		<cfparam name="ATTRIBUTES.content" type="string">
		<!--- Timestamp of edit. Exception thrown if current edit is newer. --->
		<cfparam name="ATTRIBUTES.timestamp" type="date">
	</cfcase>

	<!--- Formatting a page. --->
	<cfcase value="format">
		<!--- Text to be formatted. --->
		<cfparam name="ATTRIBUTES.text" default="" type="string">
		<!--- Name of the variable whose contents is to be formatted. --->
		<cfparam name="ATTRIBUTES.variable" type="variableName">
	</cfcase>

	<!--- Get a list of wikiwords. --->
	<cfcase value="list">
		<!--- Variable to write the list to. --->
		<cfparam name="ATTRIBUTES.variable" type="variableName">
	</cfcase>

	<cfdefaultcase>
		<cfthrow message="Invalid action: #ATTRIBUTES.action#">
	</cfdefaultcase>

</cfswitch>
<!--- END: Check parameters --->

<!--- An application variable the user can set if they want a longer to-stable time. --->
<cfparam name="APPLICATION.daysToStable" type="numeric" default="1">

<!--- Ensure all the wikiwords are cached in memory. --->
<cfif Not IsDefined("APPLICATION.wikiWordList")>
    <cflock scope="APPLICATION" type="EXCLUSIVE" timeout="10">
        <!--- Build a cache of the wiki page list --->
        <cfset wikiWordList = StructNew()>
        <cfdirectory action="LIST" directory="#ATTRIBUTES.pageDir#" filter="*.txt" name="pageList">
        <cfloop query="pageList">
            <cfset wikiWordList[Left(name, Len(name) - 4)] = Left(name, Len(name) - 4)>
        </cfloop>
        <cfdirectory action="LIST" directory="#ATTRIBUTES.pageDir#" filter="*.edit" name="pageList">
        <cfloop query="pageList">
            <cfset wikiWordList[Left(name, Len(name) - 5)] = Left(name, Len(name) - 5)>
        </cfloop>
		<cfset wikiWordList["RecentChanges"] = "RecentChanges">
		<cfset wikiWordList["PageIndex"] = "PageIndex">
		<cfset APPLICATION.wikiWordList = wikiWordList>
    </cflock>
</cfif>

<cfscript>
/*
 * Strips all bad characters from a wikiword.
 */
function SanitiseName(name)
{
	return REReplace(name, "[^[:alpha:]-]", "", "ALL");
}
</cfscript>

<!--- Process actions --->
<cfswitch expression="#ATTRIBUTES.action#">

	<!--- Retrieving the contents of a page. --->
	<cfcase value="get">
		<!--- Clean up the name and build the path. --->
		<cfset ATTRIBUTES.name = SanitiseName(ATTRIBUTES.name)>
		<cfset path = ATTRIBUTES.pageDir & ATTRIBUTES.name>

		<cfif Not StructKeyExists(APPLICATION.wikiWordList, ATTRIBUTES.name)>
			<!--- File doesn't exist: use default. --->
			<cfset timestamp = Now()>
			<cfset content   = ATTRIBUTES.default>
		<cfelse>
			<cfif FileExists("#path#.edit")>
				<!--- Ensure the stable copy is up to date. --->
			    <cflock timeout="10" name="edit:#ATTRIBUTES.name#" type="exclusive">
			        <cfdirectory action="list" directory="#ATTRIBUTES.pageDir#"
			            filter="#ATTRIBUTES.name#.edit" name="edit">
			        <cfset editLastModified = ParseDateTime(edit.dateLastModified)>
			        <cfif DateDiff("d", editLastModified, Now()) GTE APPLICATION.daysToStable>
			            <cflock timeout="10" name="stable:#ATTRIBUTES.name#" type="exclusive">
							<!--- Update, with an insurance policy... --->
			                <cffile action="MOVE" source="#path#.txt"  destination="#path#.old">
			                <cffile action="MOVE" source="#path#.edit" destination="#path#.txt">
			            </cflock>
						<cfmail to="me@example.com" from="me@example.com" subject="[Talideon.com WIKI] Page going from edit to stable">#ATTRIBUTES.name#</cfmail>
			        </cfif>
			    </cflock>
			</cfif>

			<cfif ATTRIBUTES.for EQ "display" Or Not FileExists("#path#.edit")>
				<!--- Get the stable version if displaying or no edit version. --->
				<cfset path = "#path#.txt">
			<cfelse>
				<cfset path = "#path#.edit">
			</cfif>

			<!--- Get the timestamp and the file contents. --->
	        <cfdirectory action="LIST" directory="#GetDirectoryFromPath(path)#"
	            filter="#GetFileFromPath(path)#" name="about">
	        <cfset timestamp = ParseDateTime(about.dateLastModified)>
	        <cffile action="READ" file="#path#" variable="content">
		</cfif>

		<cfif ATTRIBUTES.formatted>
			<cf_format variable="content" wikiwords="#APPLICATION.wikiWordList#">
		</cfif>

		<!--- Return it all. --->
		<cfscript>
		SetVariable("CALLER.#ATTRIBUTES.timestamp#", timestamp);
		SetVariable("CALLER.#ATTRIBUTES.variable#",  content);
		</cfscript>
	</cfcase>

	<!--- Saving the contents of a page. --->
	<cfcase value="set">
		<!--- New content for page. --->
		<cfparam name="ATTRIBUTES.content" type="string">
		<!--- Timestamp of edit. Exception thrown if  --->
		<cfparam name="ATTRIBUTES.timestamp" type="date">

		<!--- Clean up the name and build the path. --->
		<cfset ATTRIBUTES.name = SanitiseName(ATTRIBUTES.name)>
		<cfset path = ATTRIBUTES.pageDir & ATTRIBUTES.name>

		<cfif FileExists("#path#.edit")>
			<!--- Ensure the submitted page is newer than the edit copy. --->
	        <cfdirectory action="LIST" directory="#ATTRIBUTES.pageDir#"
	            filter="#ATTRIBUTES.name#.edit" name="about">
	        <cfset timestamp = ParseDateTime(about.dateLastModified)>
			<cfif DateCompare(ATTRIBUTES.timestamp, timestamp) EQ -1>
				<!--- Edit copy is newer! Evasive manoeuvres, Mr. Sulu! --->
				<cfthrow message="Edit of &laquo;#HTMLEditFormat(ATTRIBUTES.name)#&raquo; copy is newer!">
			</cfif>
		</cfif>

		<!--- Record the existence of the page. --->
		<cfset APPLICATION.wikiWordList[ATTRIBUTES.name] = ATTRIBUTES.name>

		<!--- All is well. --->
        <cffile action="WRITE" file="#path#.edit" output="#FORM.body#" addnewline="No">
		<cfif Not FileExists("#path#.txt")>
			<cffile action="WRITE" file="#path#.txt" output="#FORM.body#" addnewline="No">
		</cfif>
	</cfcase>

	<!--- Format some text for the wiki. --->
	<cfcase value="format">
		<cf_format variable="ATTRIBUTES.text" wikiwords="#APPLICATION.wikiWordList#">
		<cfscript>
		SetVariable("CALLER.#ATTRIBUTES.variable#", ATTRIBUTES.text);
		</cfscript>
	</cfcase>

	<!--- Return a comma-seperated list of all wikiwords. --->
	<cfcase value="list">
		<cfscript>
		SetVariable("CALLER.#ATTRIBUTES.variable#", StructKeyList(APPLICATION.wikiWordList));
		</cfscript>
	</cfcase>

</cfswitch>
</cfsilent>
